@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* printf.c: the echo/printf built-ins */
-/* (C) 2007-2012 magicant */
+/* (C) 2007-2019 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -134,7 +134,7 @@ int echo_builtin(int argc, void **argv)
     if (eoption) {
 	assert(noption);
 	for (index = 1; index < argc; index++) {
-	    if (ARGV(index)[0] != L'-')
+	    if (ARGV(index)[0] != L'-' || ARGV(index)[1] == L'\0')
 		break;
 	    if (ARGV(index)[wcsspn(&ARGV(index)[1], L"neE") + 1] != L'\0')
 		break;
@@ -225,15 +225,16 @@ normal:
 	} else {
 	    switch (s[1]) {
 		wchar_t c;
-		case L'a':   c = L'\a';  goto print_char;
-		case L'b':   c = L'\b';  goto print_char;
+		case L'a':   c = L'\a';    goto print_char;
+		case L'b':   c = L'\b';    goto print_char;
 		case L'c':   return PR_OK_END;
-		case L'f':   c = L'\f';  goto print_char;
-		case L'n':   c = L'\n';  goto print_char;
-		case L'r':   c = L'\r';  goto print_char;
-		case L't':   c = L'\t';  goto print_char;
-		case L'v':   c = L'\v';  goto print_char;
-		case L'\\':  c = L'\\';  goto print_char;
+		case L'e':   c = L'\033';  goto print_char;
+		case L'f':   c = L'\f';    goto print_char;
+		case L'n':   c = L'\n';    goto print_char;
+		case L'r':   c = L'\r';    goto print_char;
+		case L't':   c = L'\t';    goto print_char;
+		case L'v':   c = L'\v';    goto print_char;
+		case L'\\':  c = L'\\';    goto print_char;
 print_char:
 		    if (!sb_wccat(buf, c, st))
 			return PR_ERROR;
@@ -667,9 +668,8 @@ enum printf_result_T printf_printf(
 	    else
 		arg = L"";
 	    return printf_print_escape(format, arg, buf);
-	default:
-	    assert(false);
     }
+    assert(false);
 }
 
 /* Parses the specified string as an integer. */
