@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* history.c: command history management */
-/* (C) 2007-2015 magicant */
+/* (C) 2007-2016 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1247,6 +1247,18 @@ int fc_builtin(int argc, void **argv)
 	}
     }
 
+    /* parse <old=new> */
+    const wchar_t *old = NULL, *new = NULL;
+    if (silent && xoptind < argc) {
+	wchar_t *eq = wcschr(ARGV(xoptind), L'=');
+	if (eq != NULL) {
+	    eq[0] = L'\0';
+	    old = ARGV(xoptind);
+	    new = &eq[1];
+	    xoptind++;
+	}
+    }
+
     /* error checks */
     if (editor && list)
 	return mutually_exclusive_option_error(L'e', L'l');
@@ -1286,18 +1298,6 @@ int fc_builtin(int argc, void **argv)
 	}
     }
 
-    /* parse <old=new> */
-    const wchar_t *old = NULL, *new = NULL;
-    if (silent && xoptind < argc) {
-	wchar_t *eq = wcschr(ARGV(xoptind), L'=');
-	if (eq != NULL) {
-	    eq[0] = L'\0';
-	    old = ARGV(xoptind);
-	    new = &eq[1];
-	    xoptind++;
-	}
-    }
-
     /* parse <first> */
     const wchar_t *vfirst = (xoptind < argc) ? ARGV(xoptind++) : NULL;
     int nfirst;
@@ -1344,11 +1344,11 @@ check_rev:
 	struct search_result_T temp = lfirst;  lfirst = llast;  llast = temp;
 	rev = !rev;
     }
-    assert(lfirst.next != Histlist);
-    assert(llast.prev  != Histlist);
 
-    const histentry_T *efirst = ashistentry(lfirst.next);
-    const histentry_T *elast  = ashistentry(llast.prev);
+    const histentry_T *efirst =
+	    ashistentry(lfirst.next == Histlist ? lfirst.prev : lfirst.next);
+    const histentry_T *elast =
+	    ashistentry(llast.prev == Histlist ? llast.next : llast.prev);
     if (list)
 	return fc_print_entries(stdout, efirst, elast, rev, ptype);
     else
@@ -1812,9 +1812,10 @@ int history_read(const wchar_t *s)
     if (f == NULL)
 	goto error;
     fc_read_history(f, true);
-    if ((ferror(f) != 0) | (fclose(f) != 0))
-	goto error;
-    return Exit_SUCCESS;
+    bool error = ferror(f);
+    error |= fclose(f);
+    if (!error)
+	return Exit_SUCCESS;
 
 error:
     xerror(0, Ngt("cannot read history from file `%ls'"), s);
