@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* job.c: job control */
-/* (C) 2007-2017 magicant */
+/* (C) 2007-2019 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <string.h>
 #include <sys/wait.h>
 #include <unistd.h>
+#include <wchar.h>
 #include <wctype.h>
 #include "builtin.h"
 #include "exec.h"
@@ -173,6 +174,9 @@ void remove_all_jobs(void)
 void free_job(job_T *job)
 {
     if (job != NULL) {
+#ifndef NDEBUG
+	assert(!job->j_beingwaitedfor);
+#endif
 	for (size_t i = 0; i < job->j_pcount; i++)
 	    free(job->j_procs[i].pr_name);
 	free(job);
@@ -374,8 +378,8 @@ start:
 		    waitpidoption &= ~WCONTINUED;
 		    goto start;
 		}
-		/* falls thru! */
 #endif
+		; /* falls thru! */
 	    default:
 		xerror(errno, "waitpid");
 		return;
@@ -393,7 +397,8 @@ start:
     for (jobnumber = 0; jobnumber < joblist.length; jobnumber++)
 	if ((job = joblist.contents[jobnumber]) != NULL)
 	    for (pnumber = 0; pnumber < job->j_pcount; pnumber++)
-		if ((pr = &job->j_procs[pnumber])->pr_pid == pid)
+		if ((pr = &job->j_procs[pnumber])->pr_pid == pid &&
+			pr->pr_status != JS_DONE)
 		    goto found;
 
     /* If `pid' was not found in the job list, we simply ignore it. This may
@@ -455,8 +460,10 @@ int wait_for_job(size_t jobnumber, bool return_on_stop,
     job_T *job = joblist.contents[jobnumber];
 
     if (!job->j_legacy) {
-	bool savenonotify = job->j_nonotify;
-	job->j_nonotify = true;
+#ifndef NDEBUG
+	assert(!job->j_beingwaitedfor);
+	job->j_beingwaitedfor = true;
+#endif
 	for (;;) {
 	    if (job->j_status == JS_DONE)
 		break;
@@ -466,7 +473,9 @@ int wait_for_job(size_t jobnumber, bool return_on_stop,
 	    if (signum != 0)
 		break;
 	}
-	job->j_nonotify = savenonotify;
+#ifndef NDEBUG
+	job->j_beingwaitedfor = false;
+#endif
     }
     return signum;
 }
@@ -493,7 +502,9 @@ wchar_t **wait_for_child(pid_t cpid, pid_t cpgid, bool return_on_stop)
     job->j_status = JS_RUNNING;
     job->j_statuschanged = false;
     job->j_legacy = false;
-    job->j_nonotify = false;
+#ifndef NDEBUG
+    job->j_beingwaitedfor = false;
+#endif
     job->j_pcount = 1;
     job->j_procs[0].pr_pid = cpid;
     job->j_procs[0].pr_status = JS_RUNNING;
@@ -737,7 +748,7 @@ bool any_job_status_has_changed(void)
 {
     for (size_t i = 1; i < joblist.length; i++) {
 	const job_T *job = get_job(i);
-	if (job != NULL && !job->j_nonotify && job->j_statuschanged)
+	if (job != NULL && job->j_statuschanged)
 	    return true;
     }
     return false;
@@ -755,19 +766,18 @@ bool any_job_status_has_changed(void)
 int print_job_status(size_t jobnumber,
 	bool changedonly, bool verbose, bool remove_done, FILE *f)
 {
-    int result = 0;
-
     job_T *job = get_job(jobnumber);
-    if (job == NULL || job->j_nonotify)
-	return result;
+    if (job == NULL)
+	return 0;
     if (changedonly && !job->j_statuschanged)
-	return result;
+	return 0;
 
     char current;
     if      (jobnumber == current_jobnumber)  current = '+';
     else if (jobnumber == previous_jobnumber) current = '-';
     else                                      current = ' ';
 
+    int result;
     if (!verbose) {
 	bool needfree;
 	char *status = get_job_status_string(job, &needfree);
