@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* job.h: job control */
-/* (C) 2007-2012 magicant */
+/* (C) 2007-2019 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,13 +43,15 @@ typedef struct process_T {
 
 /* info about a job */
 typedef struct job_T {
-    pid_t       j_pgid;          /* process group ID */
+    pid_t       j_pgid;            /* process group ID */
     jobstatus_T j_status;
-    _Bool       j_statuschanged; /* job's status not yet reported? */
-    _Bool       j_legacy;        /* not a true child of the shell? */
-    _Bool       j_nonotify;      /* suppress printing job status? */
-    size_t      j_pcount;        /* # of processes in `j_procs' */
-    process_T   j_procs[];       /* info about processes */
+    _Bool       j_statuschanged;   /* job's status not yet reported? */
+    _Bool       j_legacy;          /* not a true child of the shell? */
+#ifndef NDEBUG
+    _Bool       j_beingwaitedfor;  /* wait_for_job is awaiting? */
+#endif
+    size_t      j_pcount;          /* # of processes in `j_procs' */
+    process_T   j_procs[];         /* info about processes */
 } job_T;
 /* When job control is off, `j_pgid' is 0 since the job shares the process group
  * ID with the shell.
@@ -94,6 +96,8 @@ extern void ensure_foreground(void);
 extern int calc_status_of_job(const job_T *job)
     __attribute__((pure,nonnull));
 
+extern _Bool any_job_status_has_changed(void)
+    __attribute__((pure));
 extern void print_job_status_all(void);
 extern void notify_signaled_job(size_t jobnumber);
 
