@@ -60,19 +60,18 @@ size_t xstrnlen(const char *s, size_t maxlen)
 }
 #endif
 
+#if !HAVE_STRDUP
 /* Returns a newly malloced copy of the specified string.
- * The copy is at most `len' bytes long.
- * Returns an exact copy if (strlen(s) <= len).
  * Aborts the program on malloc failure. */
-char *xstrndup(const char *s, size_t len)
+char *xstrdup(const char *s)
 {
-    len = xstrnlen(s, len);
-
+    size_t len = strlen(s);
     // char *result = xmalloce(len, 1, sizeof (char));
     char *result = xmalloc(add(len, 1));
     result[len] = '\0';
     return memcpy(result, s, len);
 }
+#endif
 
 #if !HAVE_WCSNLEN
 /* Returns min(maxlen, wcslen(s)). */
