@@ -1,6 +1,6 @@
 /* Yash: yet another shell */
 /* variable.c: deals with shell variables and parameters */
-/* (C) 2007-2018 magicant */
+/* (C) 2007-2019 magicant */
 
 /* This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 #include <wchar.h>
 #include <wctype.h>
@@ -334,7 +335,7 @@ void init_variables(void)
 	v->v_value = NULL;
 	v->v_getter = random_getter;
 	random_active = true;
-	srand(shell_pid);
+	srand((unsigned) time(NULL) ^ (unsigned) shell_pid << 17);
     } else {
 	random_active = false;
     }
@@ -462,7 +463,7 @@ void reset_locale(const wchar_t *name)
 	/* POSIX forbids resetting LC_CTYPE even if the value of the variable
 	 * is changed, but we do reset LC_CTYPE if the shell is interactive and
 	 * not in the POSIXly-correct mode. */
-	if (wcscmp(name + 3, L VAR_LC_ALL + 3) == 0) {
+	if (wcscmp(&name[3], &(L VAR_LC_ALL)[3]) == 0) {
 reset_locale_all:
 	    reset_locale_category(L VAR_LC_COLLATE, LC_COLLATE);
 	    if (!posixly_correct && is_interactive_now)
@@ -471,18 +472,18 @@ reset_locale_all:
 	    reset_locale_category(L VAR_LC_MONETARY, LC_MONETARY);
 	    reset_locale_category(L VAR_LC_NUMERIC, LC_NUMERIC);
 	    reset_locale_category(L VAR_LC_TIME, LC_TIME);
-	} else if (wcscmp(name + 3, L VAR_LC_COLLATE + 3) == 0) {
+	} else if (wcscmp(&name[3], &(L VAR_LC_COLLATE)[3]) == 0) {
 	    reset_locale_category(L VAR_LC_COLLATE, LC_COLLATE);
-	} else if (wcscmp(name + 3, L VAR_LC_CTYPE + 3) == 0) {
+	} else if (wcscmp(&name[3], &(L VAR_LC_CTYPE)[3]) == 0) {
 	    if (!posixly_correct && is_interactive_now)
 		reset_locale_category(L VAR_LC_CTYPE, LC_CTYPE);
-	} else if (wcscmp(name + 3, L VAR_LC_MESSAGES + 3) == 0) {
+	} else if (wcscmp(&name[3], &(L VAR_LC_MESSAGES)[3]) == 0) {
 	    reset_locale_category(L VAR_LC_MESSAGES, LC_MESSAGES);
-	} else if (wcscmp(name + 3, L VAR_LC_MONETARY + 3) == 0) {
+	} else if (wcscmp(&name[3], &(L VAR_LC_MONETARY)[3]) == 0) {
 	    reset_locale_category(L VAR_LC_MONETARY, LC_MONETARY);
-	} else if (wcscmp(name + 3, L VAR_LC_NUMERIC + 3) == 0) {
+	} else if (wcscmp(&name[3], &(L VAR_LC_NUMERIC)[3]) == 0) {
 	    reset_locale_category(L VAR_LC_NUMERIC, LC_NUMERIC);
-	} else if (wcscmp(name + 3, L VAR_LC_TIME + 3) == 0) {
+	} else if (wcscmp(&name[3], &(L VAR_LC_TIME)[3]) == 0) {
 	    reset_locale_category(L VAR_LC_TIME, LC_TIME);
 	}
     }
@@ -1036,25 +1037,15 @@ void random_getter(variable_T *var)
 /* Returns a random number between 0 and 32767 using `rand'. */
 unsigned next_random(void)
 {
-#if RAND_MAX == 32767
-    return (unsigned) rand();
-#elif RAND_MAX == 65535
-    return (unsigned) rand() >> 1;
-#elif RAND_MAX == 2147483647
-    return (unsigned) rand() >> 16;
-#elif RAND_MAX == 4294967295
-    return (unsigned) rand() >> 17;
+#if (RAND_MAX & (RAND_MAX + 1u)) == 0u // RAND_MAX + 1 is a power of 2
+    unsigned v = (unsigned) rand();
+    return (v ^ (v >> 15)) & ((1u << 15) - 1u);
 #else
-    unsigned rem = RAND_MAX % 32768 + 1;
-    if (rem == 32768) {
-	return (unsigned) rand() & 32767;
-    } else {
-	unsigned value;
-	do
-	    value = rand();
-	while (value > RAND_MAX - rem);
-	return value;
-    }
+    unsigned v;
+    do
+	v = (unsigned) rand();
+    while (v > RAND_MAX - (RAND_MAX + 1u) % (1u << 15));
+    return v & ((1u << 15) - 1u);
 #endif
 }
 
